#include "ShareMemUPSCmd.h"

CShareMem::CShareMem()           
{
	m_nDataCount = 0;
	m_cDataSize = 0;
	m_bInit = FALSE;
	m_bExist = FALSE;
	m_lpvData = NULL;
	m_hSharedMemoryFile = NULL;
	m_strMutantName = ""; //L"ShareMemMutant";
	m_strShareMemName = ""; //L"ShareMemSection";
}

BOOL CShareMem::Clear() 
{
	try
	{
		memset((PBYTE)m_lpvData, 0, m_cDataSize*m_nDataCount);
		return TRUE;
	}
	catch(...)
	{
		return FALSE;
	}
}
CShareMem::~CShareMem()
{
	if (m_bInit) 
	{
		UnmapViewOfFile(m_lpvData);
		CloseHandle(m_hSharedMemoryFile);
	}
}
CShareMem::CShareMem(int cCount,int cSize) 
{
	m_nDataCount = cCount;
	m_cDataSize = cSize;
	m_bInit = FALSE;
	m_lpvData = NULL;
	m_hSharedMemoryFile = NULL;
	m_bExist = FALSE;
	m_strMutantName = "";//L"ShareMemMutant";
	m_strShareMemName = "";//L"ShareMemSection";
}

LPVOID CShareMem::GetData(int nIndex )  //the nindex is the place relative at the share mem
{
	if (m_bInit)                               //CShareMem the sign of the sharemem for ??read
	{
		if (nIndex < 0 || nIndex >= m_nDataCount)
			return NULL;
		return (PBYTE)m_lpvData + nIndex * m_cDataSize;
	}
	else
	{
		return NULL;
	}
}

BOOL CShareMem::GetData(int nIndex,LPVOID lpvData,int cSize)
{
	if (m_bInit)
	{
		if (nIndex < 0 || nIndex >= m_nDataCount)
		    return FALSE;
		if (cSize != m_cDataSize )
			return FALSE;

		memcpy(lpvData,(PBYTE)m_lpvData + nIndex * m_cDataSize,cSize);
		return TRUE;
	}
	else
		return FALSE;
}

BOOL CShareMem::Delete(int nIndex)
{
	if (m_bInit)
	{
		if (nIndex < 0 || nIndex >= m_nDataCount)
		    return FALSE;
		int nLeft = m_nDataCount - nIndex - 1;
		PBYTE pbBuff = new BYTE[nLeft * m_cDataSize];
		if (0 == nIndex )
		{
			memcpy(pbBuff,(PBYTE)m_lpvData + m_cDataSize ,(m_nDataCount -1)*m_cDataSize);
			memset(m_lpvData,0,m_nDataCount * m_cDataSize);
			memcpy(m_lpvData,pbBuff,(m_nDataCount -1)*m_cDataSize);
		}
		else if ((m_nDataCount -1) == nIndex ) 
		{
			memset((PBYTE)m_lpvData + (nIndex *m_cDataSize),0 ,m_cDataSize);
		}
		else
		{
 			memcpy(pbBuff,(PBYTE)m_lpvData + (nIndex + 1)*m_cDataSize,nLeft * m_cDataSize);
			memset((PBYTE)m_lpvData + (nIndex )*m_cDataSize,0,(m_nDataCount - nIndex) * m_cDataSize);
			memcpy((PBYTE)m_lpvData + (nIndex )*m_cDataSize,pbBuff,nLeft *m_cDataSize); 
		}
		delete [] pbBuff;
		m_nDataCount --;
		return TRUE;
	}
	else
		return FALSE;
}

BOOL CShareMem::AddData(LPVOID lpvData,int cSize)
{
	try
	{
		if ((m_cDataSize!= 0 && m_cDataSize != cSize))
			return FALSE;
		if (!m_bInit)
			return FALSE;

		memcpy((PBYTE)m_lpvData + (m_nDataCount )*m_cDataSize, lpvData,cSize);
		m_nDataCount ++ ;
		return TRUE;
	}
	catch(CMemoryException *memE)
	{
		memE->ReportError();
		memE->Delete();
		return FALSE;
	}
	catch(...)
	{
		return FALSE;
	}
}

BOOL CShareMem::SetData(int nIndex,LPVOID lpvData,int cSize)
{
	if (m_bInit)
	{
		if (nIndex < 0 || nIndex >= m_nDataCount)
			return FALSE;
		if (m_cDataSize != cSize)
		{
			TRACE(_T("Size of Data Not Correct!!\n"));
			return FALSE;
		}
		memset((PBYTE)m_lpvData + nIndex * m_cDataSize,0,cSize);
		memcpy((PBYTE)m_lpvData + nIndex * m_cDataSize,lpvData,cSize);
		return TRUE;
	}
	else
		return FALSE;
}

BOOL CShareMem::InitializeMem(int cCount,int cSize, char *lpShareMemName, char *lpMutantName, BOOL bInit)
{	
	SECURITY_ATTRIBUTES secuAttr;
	SECURITY_DESCRIPTOR	sd;

	secuAttr.nLength = sizeof( SECURITY_ATTRIBUTES );
	secuAttr.bInheritHandle = FALSE;

	InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);

	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	secuAttr.lpSecurityDescriptor = &sd;

	if (lpShareMemName == NULL || lpMutantName == NULL)
		return FALSE;

	m_strMutantName = lpMutantName;
	m_strShareMemName = lpShareMemName;

	if (bInit == TRUE)
		m_hMutant = CreateMutex(&secuAttr, FALSE, m_strMutantName);
	else
	{
		m_hMutant = OpenMutex(MUTEX_ALL_ACCESS, FALSE, m_strMutantName);
		if (m_hMutant == NULL)
		{
			Sleep(1000);
			m_hMutant = OpenMutex(MUTEX_ALL_ACCESS, FALSE, m_strMutantName); 
			if (m_hMutant)
				return FALSE;
		}
	}

	m_cDataSize = cSize;
	m_nDataCount = cCount;

	m_hSharedMemoryFile = ::CreateFileMapping(
							NULL, 
							&secuAttr,              
							PAGE_READWRITE,    
							0,                   
							SHM_DATA_LENGTH * 2,              
							m_strShareMemName.GetBuffer(0)
							);


	if (m_hSharedMemoryFile == NULL)
	{
		m_bExist = FALSE;
		m_bInit = FALSE;

		return FALSE;
	}
	else
	{
		if (GetLastError() == ERROR_ALREADY_EXISTS)
			m_bExist = TRUE;
	}
	m_lpvData = ::MapViewOfFile(
				m_hSharedMemoryFile,     
				FILE_MAP_ALL_ACCESS, 
				0,                  
				0,                 
				0
				);            
	if (m_lpvData == NULL)
	{
		m_bInit = FALSE;
		CloseHandle(m_hSharedMemoryFile);

		return FALSE;
	}
	else
		m_bInit = TRUE;

	return TRUE; 
}

BOOL CShareMem::Lock(DWORD dwMilliSeconds )
{
	DWORD dwRet = 0;
	dwRet = WaitForSingleObject(m_hMutant,dwMilliSeconds);

	if (dwRet == WAIT_OBJECT_0)
		return TRUE;

	return FALSE;
}

BOOL CShareMem::Unlock()
{
	return ReleaseMutex(m_hMutant);
}


void CShareMem::ReleaseHandle()
{
	if (m_bInit) //
	{
		CloseHandle(m_hMutant);
		UnmapViewOfFile(m_lpvData);
		CloseHandle(m_hSharedMemoryFile);
		m_bInit = FALSE;
	}
}
